################################################################
##
## MySQL Database Class
from __future__ import division
import rankme.pymysql as pymysql
from rankme import rankme
import es, playerlib

class MySQLDataBase(rankme.SQLite3DataBase):
    ''' The ranking database, handles all off the data '''
    def __init__(self):
        ''' Fires when the script is initialized '''
        # Get the points_system value, so it can be accessed easily
        self.points = rankme.settings.points_system
        # Create a connection to our database
        self.connection = pymysql.connect(host=rankme.settings.hostname, user=rankme.settings.username, passwd=rankme.settings.password)
        # Establish a cursor for data handling
        self.cursor = self.connection.cursor()
        try:
            self.execute("CREATE DATABASE IF NOT EXISTS %s" % rankme.settings.db)
        except:
            pass
        
        try:
            self.execute("USE %s" % rankme.settings.db) 
        except:
            raise ValueError("[rankME] Cannot connect to database %s and do not have permission to create either" % rankme.settings.db)
        
        self.execute("""\
        CREATE TABLE IF NOT EXISTS online (
        steamid VARCHAR(30) PRIMARY KEY NOT NULL,
        name VARCHAR(100),
        ip VARCHAR(100),
        kills INTEGER DEFAULT 0,
        deaths INTEGER DEFAULT 0,
        headshots INTEGER DEFAULT 0,
        timeplayed INTEGER DEFAULT 0,
        points INTEGER DEFAULT 0,
        team INTEGER DEFAULT 0,
        damage INTEGER DEFAULT 0, 
        tWins INTEGER DEFAULT 0, 
        ctWins INTEGER DEFAULT 0, 
        ctTotal INTEGER DEFAULT 0, 
        tTotal INTEGER DEFAULT 0, 
        bP INTEGER DEFAULT 0, 
        bD INTEGER DEFAULT 0, 
        hR INTEGER DEFAULT 0, 
        hK INTEGER DEFAULT 0, 
        bE INTEGER DEFAULT 0, 
        suicides INTEGER DEFAULT 0
        )""")
        
        self.execute("""\
        CREATE TABLE IF NOT EXISTS data (
        steamid VARCHAR(30) PRIMARY KEY NOT NULL, 
        name VARCHAR(100), 
        kills INTEGER DEFAULT 0, 
        deaths INTEGER DEFAULT 0, 
        damage INTEGER DEFAULT 0, 
        tWins INTEGER DEFAULT 0, 
        ctWins INTEGER DEFAULT 0, 
        ctTotal INTEGER DEFAULT 0, 
        tTotal INTEGER DEFAULT 0, 
        bP INTEGER DEFAULT 0, 
        bD INTEGER DEFAULT 0, 
        hR INTEGER DEFAULT 0, 
        hK INTEGER DEFAULT 0, 
        bE INTEGER DEFAULT 0, 
        suicides INTEGER DEFAULT 0, 
        tP INTEGER DEFAULT 0, 
        hs INTEGER DEFAULT 0, 
        points INTEGER DEFAULT 0
        )""")
        
        self.execute("""\
        CREATE TABLE IF NOT EXISTS hitbox (
        steamid VARCHAR(30) PRIMARY KEY NOT NULL,
        head INTEGER DEFAULT 0, 
        chest INTEGER DEFAULT 0, 
        leftarm INTEGER DEFAULT 0, 
        rightarm INTEGER DEFAULT 0, 
        stomach INTEGER DEFAULT 0, 
        leftleg INTEGER DEFAULT 0, 
        rightleg INTEGER DEFAULT 0
        )""")
        
        self.execute("""\
        CREATE TABLE IF NOT EXISTS userInformation (
        playerid INTEGER AUTO_INCREMENT NOT NULL PRIMARY KEY,
        steamid VARCHAR(30) NOT NULL,
        ip VARCHAR(100) NOT NULL,
        created TIMESTAMP DEFAULT NOW(),
        lastconnect TIMESTAMP,
        authcode VARCHAR(200),
        connects INTEGER DEFAULT 0,
        realname VARCHAR(100),
        email VARCHAR(100),
        homepage VARCHAR(100)
        )""")
        
        #### MAP TABLE
        
        self.execute("""\
        CREATE TABLE IF NOT EXISTS maps (
        steamid VARCHAR(30) NOT NULL,
        map VARCHAR(150),
        time INTEGER DEFAULT 0,
        kills INTEGER DEFAULT 0,
        deaths INTEGER DEFAULT 0,
        headshots INTEGER DEFAULT 0,
        points INTEGER DEFAULT 0
        )""")
        
        #### NAMES TABLE
        
        self.execute("""\
        CREATE TABLE IF NOT EXISTS names (
        steamid VARCHAR(30) PRIMARY KEY NOT NULL,
        name VARCHAR(100)
        )""")

        string = "CREATE TABLE IF NOT EXISTS weapons (steamid VARCHAR(30) PRIMARY KEY NOT NULL"
        for weapon in rankme.all_weapons:
            string += ', %s INTEGER DEFAULT 0' % weapon
            if weapon.startswith('smoke'):
                string += ')'
        self.execute(string)
        
        
    ##
    ## USER EXISTENCE CHECK FUNCTIONS
    ##
       
    def __contains__(self, steamid):
        ''' If the script contains the players steamid then return True otherwise False '''
        return bool(self.fetchone('SELECT steamid FROM data WHERE steamid="%s"'%steamid))
       
    def hitCheck(self, steamid):
        ''' Checks whether the player is in the hitbox database '''
        return bool(self.fetchone('SELECT steamid FROM hitbox WHERE steamid="%s"'%steamid))
        
    def weaponCheck(self, steamid):
        ''' Checks whether the player is in the weapons database '''
        return bool(self.fetchone('SELECT steamid FROM weapons WHERE steamid="%s"'%steamid))
        
    def nameCheck(self, steamid):
        ''' Checks whether the player is in the names database '''
        return bool(self.fetchone('SELECT steamid FROM names WHERE steamid="%s"'%steamid))
    
    def onlineCheck(self, steamid):
        ''' Checks whether the player is in the names database '''
        return bool(self.fetchone('SELECT steamid FROM online WHERE steamid="%s"'%steamid))
        
    def infoCheck(self, steamid):
        ''' Checks whether the player is in the user information database '''
        return bool(self.fetchone('SELECT steamid FROM userInformation WHERE steamid="%s"'%steamid))
        
    def mapCheck(self, steamid, map):
        ''' Checks whether the player is in the map information database '''
        return bool(self.fetchone('SELECT steamid FROM maps WHERE steamid="%s" AND map="%s"'%(steamid, map)))
        
    ##
    ## GET FUNCTIONS
    ##
    
    def addMapToPlayer(self, steamid, map):
        ''' Adds the player to the map database '''
        if es.isbot(es.getuserid(steamid)): return
        if not self.mapCheck(steamid, map):
            self.execute('INSERT INTO maps (steamid, map) VALUES ("%s", "%s")' % (steamid, map))
        
    def getPlayerName(self, steamid):
        ''' Gets the players name from the database '''
        if not steamid: return
        if not self.nameCheck(steamid): self.setPlayerName(steamid, es.getplayername(es.getuserid(steamid)))
        result = self.fetchone("SELECT name FROM names WHERE steamid='%s'"%steamid)
        if not result:
            return None
          
        return result[0]
        
    def getBotName(self, steamid):
        ''' Gets the bots name from the database '''
        if not steamid: return
        if steamid in self:
            result = self.fetchone("SELECT name FROM data WHERE steamid='%s'"%steamid)
            if not result:
                return None
              
            return result[0]
       
    def getDamage(self, steamid):
        ''' Gets the players damage '''
        if not steamid: return
        result = self.fetchone("SELECT damage FROM data WHERE steamid='%s'"%steamid)
        if not result:
            return None
          
        return result
 
    def getHitboxData(self, steamid):
        ''' Gets all of the players hitbox data (head hits, chest hits...) '''
        if not steamid: return
        result = self.fetchone("SELECT head, chest, stomach, leftarm, rightarm, leftleg, rightleg FROM hitbox WHERE steamid='%s'"%steamid)
        if not result:
            return None
          
        return result
    
    def getTop10WeaponData(self, steamid):
        ''' Returns the weapons for the user '''
        if not steamid: return
        result = self.fetchone("SELECT * FROM weapons WHERE steamid='%s' DESC LIMIT 10"%steamid)
        if not result:
            return None
          
        return result
    
    def getWeaponData(self, steamid):
        ''' Returns the weapons for the user '''
        if not steamid: return
        result = self.fetchone("SELECT * FROM weapons WHERE steamid='%s'"%steamid)
        if not result:
            return None
          
        return result
       
    def getTotalCount(self):
        ''' Get the total count of players in the database '''
        result = self.fetchone("SELECT count(*) FROM data")
        if result:
            return result[0]
              
        return None
    
    def getPoints(self, steamid):
        ''' Gets the players total points '''
        if not steamid: return
        result = self.fetchone("SELECT points FROM data WHERE steamid='%s'"%steamid)
        if not result:
            return None
          
        return result[0]
    
    def getKills(self, steamid):
        ''' Gets the players total kills '''
        if not steamid: return
        result = self.fetchone("SELECT kills FROM data WHERE steamid='%s'"%steamid)
        if not result:
            return None
          
        return result[0]
    
    def getAll(self):
        ''' Get all of the players in the database '''
        if not self.points:
            result = self.fetchall("SELECT steamid, kills, deaths FROM data ORDER BY kills DESC LIMIT %s" % self.getTotalCount())
        else:
            result = self.fetchall("SELECT steamid, kills, deaths, points FROM data ORDER BY points DESC LIMIT %s" % self.getTotalCount())
        
        if not result:
            return None
        
        return result
    
    def getTop(self, top):
        ''' Get the specified players in the database '''
        if self.points:
            result = self.fetchall("SELECT steamid, kills, deaths, points FROM data ORDER BY points DESC LIMIT %s OFFSET %s" % (10, top))
            
        if not result:
            return None
        
        return result
    
    def getPosition(self, steamid):
        ''' Get the players position (rank) '''
        if not steamid: return
        self.cursor.execute("SELECT count(*) FROM data WHERE points < (SELECT points FROM data WHERE steamid='%s')" % steamid)
        result = self.cursor.fetchone()
        if result:
            return self.getTotalCount() - result[0]
            
    def getPlayerTime(self, steamid):
        ''' Gets the players game time '''
        if not steamid: return
        result = self.fetchone("SELECT tP FROM data WHERE steamid='%s'"%steamid)
        if not result:
            return None
          
        return result[0]
    
    ##
    ## REMOVE FUNCTIONS
    ##
    
    def removePlayerTracker(self, steamid):
        ''' Removes the player from the online database so he is not tracked anymore '''
        if not steamid: return
        if self.onlineCheck(steamid):
            # Remove the player from the database
            self.execute('DELETE FROM online WHERE steamid="%s"' % steamid)
            
    def removeAllTrackers(self):
        ''' Removes all the players from the online database '''
        self.execute('TRUNCATE TABLE online')
        
    ##
    ## SET FUNCTIONS
    ##
    
    def setPlayerTeam(self, userid, steamid):
        ''' Sets the player's team '''
        if not steamid: return
        if not userid: return
        if not self.onlineCheck(steamid):
            self.addPlayerTracker(userid, steamid)
            
        self.execute('UPDATE online SET team=%s WHERE steamid="%s"' % (es.getplayerteam(userid), steamid))
            
    def setPlayerName(self, steamid, name):
        if not steamid: return
        if self.nameCheck(steamid):
            # Add points to his current value
            self.execute('UPDATE names SET name="%s" WHERE steamid="%s"' % (name, steamid))
        
        # If he isnt in the database..
        else:
            # Dumb method, might change in future
            if steamid == 'BOT':
                steamid == name
            # Set his variables
            self.execute('INSERT INTO names VALUES ("%s", "%s")' % (steamid, name))
            
    ##
    ## ADD FUNCTIONS
    ##
    
    def addPlayerInformation(self, steamid, userid=None):
        ''' Adds the player to the user information database to display on the website '''
        if not userid:
            userid = es.getuserid(steamid)
        if not steamid: return
        if not self.infoCheck(steamid):
            # Add the player to the user information database
            self.execute('INSERT INTO userInformation (steamid, connects, ip) VALUES ("%s", %s, "%s")' % (steamid, 0, rankme.getPlayer(userid).ip))
            
    
    def addPlayerTracker(self, userid, steamid):
        ''' Adds the player to the online database to display on the website '''
        if not steamid: return
        if not userid: return
        if not self.onlineCheck(steamid):
            # Add the player to the online database
            ip = rankme.getPlayer(userid).ip if rankme.getPlayer(userid).ip != None else steamid
            if ip: 
                self.execute('INSERT INTO online VALUES ("%s", "%s", "%s", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)' % (steamid, self.getPlayerName(steamid), ip))
            
    def addPlayerSessionTime(self, steamid):
        ''' Adds to the players online time '''
        if not steamid: return
        if self.onlineCheck(steamid):
            # Add a second to the user's total game time
            self.execute('UPDATE online SET timeplayed=timeplayed+1 WHERE steamid="%s"' % steamid)
            
    def addPlayerConnect(self, steamid):
        ''' Adds to the players total connect count '''
        if not steamid: return
        if not self.infoCheck(steamid):
            self.addPlayerInformation(steamid)
            
        self.execute('UPDATE userInformation SET connects=connects+1 WHERE steamid="%s"' % steamid)
        self.execute('UPDATE userInformation SET lastconnect=NOW() WHERE steamid="%s"' % steamid)
    
    def addWeaponKill(self, steamid, weapon):
        ''' Add the weapon kill to the players weapon statistics '''
        if not steamid: return
        if self.weaponCheck(steamid):
            # Add points to his current value
            self.execute('UPDATE weapons SET %s=%s+1 WHERE steamid="%s"' % (weapon, weapon, steamid))
        
        # If he isnt in the database..
        else:
            # And set his variables
            self.execute('INSERT INTO weapons VALUES ("%s", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)' % steamid)

    
    def addTime(self, steamid, time):
        if not steamid and not time: return
        
        if self.mapCheck(steamid, rankme.CURRENTMAP):
            self.execute('UPDATE maps SET time=time+%s WHERE steamid="%s" AND map="%s"' % (time, steamid, rankme.CURRENTMAP))
        
        if steamid in self:
            # Add points to his current value
            self.execute('UPDATE data SET tP=tP+%s WHERE steamid="%s"' % (time, steamid))
            
        # If he isnt in the database..
        else:
            # Get his name
            if steamid.startswith('BOT_'):
                name = steamid.replace('BOT_', '')
            else:
                name = es.getplayername(es.getuserid(steamid))
                  
            # And set his variables
            self.execute('INSERT INTO data VALUES ("%s", "%s", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, "%s", 0, %s)' % (steamid, name, time, int(rankme.settings.starting_points)))
     
    def addPoints(self, steamid, points):
        ''' Adds points to the player '''
        if not steamid and not points: return
        # If the point system is enabled
        if self.points:
            # If he is in the database
            if steamid in self:
                # Add points to his current value
                self.execute('UPDATE data SET points=points+%s WHERE steamid="%s"' % (points, steamid))
    
            # If he isnt in the database..
            else:
                # Get his name
                if steamid.startswith('BOT_'):
                    name = steamid.replace('BOT_', '')
                else:
                    name = es.getplayername(self.getPlayerName(steamid))
                   
                # And set his variables
                self.execute('INSERT INTO data VALUES ("%s", "%s", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 , 0, 0, 0, 0, 0, "%s")' % (steamid, name, int(rankme.settings.starting_points)))
                
            if self.onlineCheck(steamid):
                self.execute('UPDATE online SET points=points+%s WHERE steamid ="%s"' % (points, steamid))
                
            if self.mapCheck(steamid, rankme.CURRENTMAP):
                self.execute('UPDATE maps SET points=points+%s WHERE steamid="%s" AND map="%s"' % (points, steamid, rankme.CURRENTMAP))
                
    def addDefaultPoints(self, steamid):
        if not steamid: return
        if self.points:
            # If he is in the database
            if steamid not in self:
                # Get his name
                name = self.getPlayerName(steamid)
                   
                # And set his variables
                self.execute('INSERT INTO data VALUES ("%s", "%s", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 , 0, 0, 0, 0, 0, "%s")' % (steamid, name, int(rankme.settings.starting_points)))
                
    ##
    ## GAMEPLAY EVENT HANDLING FUNCTIONS
    ##
    
    def roundend(self, winner):
        ''' Updated the database at the end of the round '''
          
        # For each userid..
        for userid in es.getUseridList():
            # Get the team of the user
            team = es.getplayerteam(userid)
            # If the player is a bot
            steamid = rankme.getPlayer(userid).getUniqueId()
            if not steamid: continue
            # If he is not in the database
            if not steamid in self:
                # Get his name
                name = es.getplayername(userid)
                # Apply the data into the database
                self.execute('INSERT INTO data VALUES ("%s", "%s", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, "%s")' % (steamid, name, int(rankme.settings.starting_points)))
                     
            # If he is a Terrorist
            if team == 2:
                # Set his total round + 1
                self.execute('UPDATE data SET tTotal=tTotal+1 WHERE steamid="%s"' % steamid)
                # Set his current season rounds + 1
                self.execute('UPDATE online SET tTotal=tTotal+1 WHERE steamid="%s"' % steamid)
                # If Terrorists were the winners...
                if winner == 2:
                    # Add a point to his total wins
                    self.execute('UPDATE data SET tWins=tWins+1 WHERE steamid="%s"' % steamid)
                    # Add a point to his current season wins
                    self.execute('UPDATE online SET tWins=tWins+1 WHERE steamid="%s"' % steamid)
                    if self.points:
                        # Add points
                        self.addPoints(steamid, rankme.settings.round_end_winner_bonus)
                    
            # If the player is a CT
            elif team == 3:
                # Set his total rounds + 1
                self.execute('UPDATE data SET ctTotal=ctTotal+1 WHERE steamid="%s"' % steamid)
                # Set his current season rounds + 1
                self.execute('UPDATE online SET ctTotal=ctTotal+1 WHERE steamid="%s"' % steamid)
                # If CTs were the winners
                if winner == 3:
                    # Update the database
                    self.execute('UPDATE data SET ctWins=ctWins+1 WHERE steamid="%s"' % steamid)
                    # Add a point to his current season wins
                    self.execute('UPDATE online SET ctWins=ctWins+1 WHERE steamid="%s"' % steamid)
                    # Add points...
                    self.addPoints(steamid, rankme.settings.round_end_winner_bonus)
                      
    def hurt(self, attacker, damage, userid):
        ''' Updates the database when a player is hurt '''
        if attacker:
            # If the attacker is in the database
            if attacker in self:
                # Update the database with damage
                self.execute('UPDATE data SET damage=damage+%s WHERE steamid="%s"' % (damage, attacker))
                self.execute('UPDATE online SET damage=damage+%s WHERE steamid="%s"' % (damage, attacker))
            # If he isn't...
            else:
                # Get his name
                name = self.getPlayerName(rankme.getPlayer(userid).getUniqueId())
                  
                # Set his table details
                self.execute('INSERT INTO data VALUES ("%s", "%s", 0, 0, "%s", 0, 0, 0, 0, 0, 0, 0 , 0, 0, 0, 0, 0, "%s")' % (attacker, name, damage, int(rankme.settings.starting_points)))
           
       
    def hitbox(self, attacker, hit):
        ''' Updated the 'hitbox' database when a player is hurt '''
        if not attacker: return
        # If the player isn't in the database
        if not self.hitCheck(attacker):
            # Create him...
            self.execute('INSERT INTO hitbox VALUES ("%s", 0, 0, 0, 0, 0, 0, 0)' % attacker)
        
        # If it's a headshot
        if hit == 1:
            # Update the database...
            self.execute('UPDATE hitbox SET head=head+1 WHERE steamid="%s"' % attacker)
            
        # If it's a chest hit
        elif hit == 2:
            # Update the database...
            self.execute('UPDATE hitbox SET chest=chest+1 WHERE steamid="%s"' % attacker)
            
        # If it's a stomach hit
        elif hit == 3:
            # Update the database...
            self.execute('UPDATE hitbox SET stomach=stomach+1 WHERE steamid="%s"' % attacker)
            
        # If it's a left arm hit
        elif hit == 4:
            # Update the database...
            self.execute('UPDATE hitbox SET leftarm=leftarm+1 WHERE steamid="%s"' % attacker)
            
        # If it's a right arm hit
        elif hit == 5:
            # Update the database...
            self.execute('UPDATE hitbox SET rightarm=rightarm+1 WHERE steamid="%s"' % attacker)
            
        # If it's a left leg hit
        elif hit == 6:
            # Update the database...
            self.execute('UPDATE hitbox SET leftleg=leftleg+1 WHERE steamid="%s"' % attacker)
            
        # If it's a right leg hit
        elif hit == 7:
            # Update the database...
            self.execute('UPDATE hitbox SET rightleg=rightleg+1 WHERE steamid="%s"' % attacker)
               
   
    def death(self, victim, attacker, weapon, headshot=False, victim_userid=None, attacker_userid=None):
        ''' Updated the database when a player dies '''
        # If the attacker is equal to the victim
        if attacker_userid == victim_userid:
            # If he exists...
            if attacker in self:
                # He did a sucide
                self.execute('UPDATE data SET suicides=suicides+1 WHERE steamid="%s"' % attacker)
               
            # If he isn't in the database
            else:
                # Get his name
                name = self.getPlayerName(rankme.getPlayer(attacker_userid).getUniqueId())
                # Create him
                self.execute('INSERT INTO data VALUES ("%s", "%s", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 , 0, 0, 1, 0, 0, "%s")' % (attacker, name, int(rankme.settings.starting_points)))
                
            self.execute('UPDATE online SET suicides=suicides+1 WHERE steamid="%s"' % attacker)
            
        if not attacker: return
        if not victim: return
        
        # If the attacker is not equal to the victim and the attacker exists (world check)
        if attacker != victim and attacker:
            if es.getplayerteam(attacker_userid) == es.getplayerteam(victim_userid):
                self.execute('UPDATE data SET points=points-%s WHERE steamid="%s"' % (rankme.settings.teamkill_deduct, attacker))
                self.execute('UPDATE online SET points=points-%s WHERE steamid="%s"' % (rankme.settings.teamkill_deduct, attacker))
                index = playerlib.getPlayer(victim_userid).attributes['index']
                if index:
                    points = self.getPoints(attacker)
                    rankme.tell(attacker_userid, index, 'teamkill message', {'player' : es.getplayername(victim_userid), 'bonus' : rankme.settings.teamkill_deduct, 'points' : rankme.getFormat(points)})
                return
            
            # If he exists...
            if attacker in self:
                # Update his kills
                self.execute('UPDATE data SET kills=kills+1 WHERE steamid="%s"' % attacker)
                self.execute('UPDATE online SET kills=kills+1 WHERE steamid="%s"' % attacker)
                if self.mapCheck(attacker, rankme.CURRENTMAP):
                    self.execute('UPDATE maps SET kills=kills+1 WHERE steamid="%s" AND map="%s"' % (attacker, rankme.CURRENTMAP))
            
            # If he doesn't exist
            else:
                # Get his name
                name = self.getPlayerName(rankme.getPlayer(attacker_userid).getUniqueId())
                # Update the database
                self.execute('INSERT INTO data VALUES ("%s", "%s", 1, 0, 0, 0, 0, 0, 0, 0, 0, 0 , 0, 0, 0, 0, 0, "%s")' % (attacker, name, int(rankme.settings.starting_points)))
                
            # If it was a headshot
            if headshot:
                # Add a point to his headshot kills
                self.execute('UPDATE data SET hs=hs+1 WHERE steamid="%s"' % attacker)
                self.execute('UPDATE online SET headshots=headshots+1 WHERE steamid="%s"' % attacker)
                if self.mapCheck(attacker, rankme.CURRENTMAP):
                    self.execute('UPDATE maps SET headshots=headshots+1 WHERE steamid="%s" AND map="%s"' % (attacker, rankme.CURRENTMAP))
                
        # If the victim exists
        if victim in self:
            # Add 1 to his deaths
            self.execute('UPDATE data SET deaths=deaths+1 WHERE steamid="%s"' % victim)
            self.execute('UPDATE online SET deaths=deaths+1 WHERE steamid="%s"' % victim)
            if self.mapCheck(victim, rankme.CURRENTMAP):
                self.execute('UPDATE maps SET deaths=deaths+1 WHERE steamid="%s" AND map="%s"' % (victim, rankme.CURRENTMAP))
           
        # If he doesnt..
        else:
            # Get his name
            name = self.getPlayerName(rankme.getPlayer(victim_userid).getUniqueId())
            # Create him
            self.execute('INSERT INTO data VALUES ("%s", "%s", 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, "%s")' % (victim, name, int(rankme.settings.starting_points)))
            
        # If the points system is running
        if self.points:
            if attacker != victim and attacker:
                # Get the victim points
                victim_points = self.getPoints(victim)
                # Get the attacker points
                attacker_points = self.getPoints(attacker)
                # Do the formula
                bonus = int((victim_points/attacker_points)*rankme.settings.multiplier*float(rankme.weapon_convert[weapon]))
                if bonus == 0:
                    bonus = 1
                     
                # If it was a headshot
                if headshot:
                    # Add the headshot bonus
                    bonus = bonus + rankme.settings.headshot_bonus
               
                # Add the number of points he earned
                self.addPoints(attacker, bonus)
               
                if victim_userid and attacker_userid:
                    index = playerlib.getPlayer(victim_userid).attributes['index']
                    point = self.getPoints(attacker)
                    victim_point = self.getPoints(victim)
                    if not headshot:
                        if attacker_userid: rankme.tell(attacker_userid, index, 'player death no headshot', {'attacker' : self.getPlayerName(rankme.getPlayer(attacker_userid).getUniqueId()), 'points' : rankme.getFormat(point), 'bonus' : str(bonus), 'victim' : self.getPlayerName(rankme.getPlayer(victim_userid).getUniqueId()), 'victimpoints' : rankme.getFormat(victim_point)})
                    else:
                        if attacker_userid: rankme.tell(attacker_userid, index, 'player death with headshot', {'attacker' : self.getPlayerName(rankme.getPlayer(attacker_userid).getUniqueId()), 'points' : rankme.getFormat(point), 'bonus' : str(bonus), 'hs_bonus' : rankme.settings.headshot_bonus, 'victim' : self.getPlayerName(rankme.getPlayer(victim_userid).getUniqueId()), 'victimpoints' : rankme.getFormat(victim_point)})
                                
                index = playerlib.getPlayer(attacker_userid).attributes['index']
                point = self.getPoints(attacker)
                victim_point = self.getPoints(victim)
                if not headshot:
                    if victim_userid: rankme.tell(victim_userid, index, 'player death victim notify no headshot', {'attacker' : self.getPlayerName(rankme.getPlayer(attacker_userid).getUniqueId()), 'points' : rankme.getFormat(point), 'bonus' : str(bonus), 'victim' : self.getPlayerName(rankme.getPlayer(victim_userid).getUniqueId()), 'victimpoints' : rankme.getFormat(victim_point)})
                else:
                    if victim_userid: rankme.tell(victim_userid, index, 'player death victim notify with headshot', {'attacker' : self.getPlayerName(rankme.getPlayer(attacker_userid).getUniqueId()), 'points' : rankme.getFormat(point), 'bonus' : str(bonus), 'hs_bonus' : rankme.settings.headshot_bonus, 'victim' : self.getPlayerName(rankme.getPlayer(victim_userid).getUniqueId()), 'victimpoints' : rankme.getFormat(victim_point)})
            
        if weapon in rankme.all_weapons:
            self.addWeaponKill(attacker, weapon)
    
    def hostage(self, steamid, reason, userid):
        ''' Updated the database when a hostage event occurs '''
        # 0 = Killed, 1 = Rescued
        
        # If the hostage was killed,
        # Add the player to the database if he doesnt already exist
        # and update his hostage kills.
        if steamid:
            if reason == 0:
                if steamid in self:
                    self.execute('UPDATE data SET hK=hK+1 WHERE steamid="%s"' % steamid)
                else:
                    # Get his name
                    name = self.getPlayerName(rankme.getPlayer(userid).getUniqueId())
                    
                    self.execute('INSERT INTO data VALUES ("%s", "%s", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, "%s")' % (steamid, name, int(rankme.settings.starting_points)))
                    
                self.execute('UPDATE online SET hK=hK+1 WHERE steamid="%s"' % steamid)
                   
           # If the hostage was rescued,
           # Add the player to the database if he doesnt already exist
           # and update his hostage rescued.    
            elif reason == 1:
                if steamid in self:
                    self.execute('UPDATE data SET hR=hR+1 WHERE steamid="%s"' % steamid)
                else:
                    # Get his name
                    name = self.getPlayerName(rankme.getPlayer(userid).getUniqueId())
                    
                    self.execute('INSERT INTO data VALUES ("%s", "%s", 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, "%s")' % (steamid, name, int(rankme.settings.starting_points)))
                    
                # Add the bonus hostage rescue points
                self.execute('UPDATE online SET hR=hR+1 WHERE steamid="%s"' % steamid)
                self.addPoints(steamid, rankme.settings.hostage_rescue_bonus)
               
                # If we should notify the player of his gain of points
                if rankme.settings.message_player_events:
                    if not steamid.startswith('BOT_'):
                        index = playerlib.getPlayer(userid).attributes['index']
                        points = self.getPoints(steamid)
                        rankme.tell(userid, index, 'hostage rescue', {'bonus' : rankme.settings.hostage_rescue_bonus, 'points' : rankme.getFormat(points)})
               
               
    def bomb(self, steamid, reason, userid):
        ''' Updated the database when a bomb event occurs '''
        # 0 = Planted, 1 = Defused, 2 = Exploded
        if not steamid: return
        # If the bomb was planted,
        # Add the player to the database if he doesnt already exist
        # and update his bomb plants and add the bonus points.
        if reason == 0:
            if steamid in self:
                self.execute('UPDATE data SET bP=bP+1 WHERE steamid="%s"' % steamid)
            else:
                # Get his name
                name = self.getPlayerName(rankme.getPlayer(userid).getUniqueId())
                
                self.execute('INSERT INTO data VALUES ("%s", "%s", 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, "%s")' % (steamid, name, int(rankme.settings.starting_points)))
                
            self.execute('UPDATE online SET bP=bP+1 WHERE steamid="%s"' % steamid)
            self.addPoints(steamid, rankme.settings.bomb_plant_bonus)
           
            # If we should notify the player of his gain of points
            if rankme.settings.message_player_events:
                if not steamid.startswith('BOT_'):
                    index = playerlib.getPlayer(userid).attributes['index']
                    points = self.getPoints(steamid)
                    rankme.tell(userid, index, 'bomb planted', {'bonus' : rankme.settings.bomb_plant_bonus, 'points' : rankme.getFormat(points)}) 
                
        # If the bomb was defused,
        # Add the player to the database if he doesnt already exist
        # and update his bomb defuses and add the bonus points.
        elif reason == 1:
            if steamid in self:
                self.execute('UPDATE data SET bD=bD+1 WHERE steamid="%s"' % steamid)
            else:
                # Get his name
                name = self.getPlayerName(rankme.getPlayer(userid).getUniqueId())
               
                self.execute('INSERT INTO data VALUES ("%s", "%s", 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, "%s")' % (steamid, name, int(rankme.settings.starting_points)))
                
            self.execute('UPDATE online SET bD=bD+1 WHERE steamid="%s"' % steamid)
            self.addPoints(steamid, rankme.settings.bomb_defuse_bonus)
           
            # If we should notify the player of his gain of points
            if rankme.settings.message_player_events:
                if not steamid.startswith('BOT_'):
                    index = playerlib.getPlayer(userid).attributes['index']
                    points = self.getPoints(steamid)
                    rankme.tell(userid, index, 'bomb defused', {'bonus' : rankme.settings.bomb_defuse_bonus, 'points' : rankme.getFormat(points)}) 
                
        # If the bomb exploded,
        # Add the player to the database if he doesnt already exist
        # and update his bomb explode count and add the bonus points.
        elif reason == 2:
            if steamid in self:
                self.execute('UPDATE data SET bE=bE+1 WHERE steamid="%s"' % steamid)
            else:
                # Get his name
                name = self.getPlayerName(rankme.getPlayer(userid).getUniqueId())
               
                self.execute('INSERT INTO data VALUES ("%s", "%s", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, "%s")' % (steamid, name, int(rankme.settings.starting_points)))
                
            self.execute('UPDATE online SET bE=bE+1 WHERE steamid="%s"' % steamid)
            self.addPoints(steamid, rankme.settings.bomb_explode_bonus)
            
            # If we should notify the player of his gain of points
            if rankme.settings.message_player_events:
                if not steamid.startswith('BOT_'):
                    index = playerlib.getPlayer(userid).attributes['index']
                    points = self.getPoints(steamid)
                    rankme.tell(userid, index, 'bomb exploded', {'bonus' : rankme.settings.bomb_explode_bonus, 'points' : rankme.getFormat(points)}) 
                    
def load():
    rankme.database = MySQLDataBase()
    rankme.DATABASE_METHOD = 'MySQL'